@@ -38,12 +38,10 @@ import operator
 import re
 
 from . import errors
-from ._utils import parse_timedelta
+from ._utils import parse_datetime, parse_float, parse_timedelta
 from .suggestions import suggest_symbol
 from .types import *
 
-import dateutil.parser
-
 def _assert_is_integer_number(*values):
 	if not all(map(is_integer_number, values)):
 		raise errors.EvaluationError('data type mismatch (not an integer number)')
@@ -62,7 +60,7 @@ def _assert_is_string(*values):
 
 def _is_reduced(*values):
 	"""
-	Check if the ast expression *value* is a literal expression and if it is a compound datatype, that all of it's
+	Check if the ast expression *value* is a literal expression and if it is a compound datatype, that all of its
 	members are reduced literals. A value that causes this to evaluate to True for is able to be evaluated without a
 	*thing*.
 	"""
@@ -75,7 +73,7 @@ def _iterable_member_value_type(value):
 	return iterable_member_value_type(value)
 
 class Assignment(object):
-	"""An internal assignment where by a symbol is populated with a value of the specified type."""
+	"""An internal assignment whereby a symbol is populated with a value of the specified type."""
 	__slots__ = ('name', 'value', 'value_type')
 	def __init__(self, name, *, value=errors.UNDEFINED, value_type=None):
 		"""
@@ -120,6 +118,17 @@ class ASTNodeBase(object):
 		"""
 		return self
 
+class Comment(ASTNodeBase):
+	__slots__ = ('value',)
+	def __init__(self, value):
+		self.value = value
+
+	def __repr__(self):
+		return "<{0} {1!r}>".format(self.__class__.__name__, self.value)
+
+	def to_graphviz(self, digraph, *args, **kwargs):
+		digraph.node(str(id(self)), "{}\n{!r}".format(self.__class__.__name__, self.value))
+
 ################################################################################
 # Base Expression Classes
 ################################################################################
@@ -230,12 +239,7 @@ class DatetimeExpression(LiteralExpressionBase):
 	result_type = DataType.DATETIME
 	@classmethod
 	def from_string(cls, context, string):
-		try:
-			dt = dateutil.parser.isoparse(string)
-		except ValueError:
-			raise errors.DatetimeSyntaxError('invalid datetime', string)
-		if dt.tzinfo is None:
-			dt = dt.replace(tzinfo=context.default_timezone)
+		dt = parse_datetime(string, default_timezone=context.default_timezone)
 		return cls(context, dt)
 
 class TimedeltaExpression(LiteralExpressionBase):
@@ -247,11 +251,7 @@ class TimedeltaExpression(LiteralExpressionBase):
 	result_type = DataType.TIMEDELTA
 	@classmethod
 	def from_string(cls, context, string):
-		try:
-			dt = parse_timedelta(string)
-		except ValueError:
-			raise errors.TimedeltaSyntaxError('invalid timedelta', string)
-		return cls(context, dt)
+		return cls(context, parse_timedelta(string))
 
 class FloatExpression(LiteralExpressionBase):
 	"""Literal float expressions representing numerical values."""
@@ -260,6 +260,10 @@ class FloatExpression(LiteralExpressionBase):
 		value = coerce_value(value)
 		super(FloatExpression, self).__init__(context, value, **kwargs)
 
+	@classmethod
+	def from_string(cls, context, string):
+		return cls(context, parse_float(string))
+
 class MappingExpression(LiteralExpressionBase):
 	"""Literal mapping expression representing a set of associations between keys and values."""
 	result_type = DataType.MAPPING
@@ -1023,10 +1027,82 @@ class SymbolExpression(ExpressionBase):
 	def to_graphviz(self, digraph, *args, **kwargs):
 		digraph.node(str(id(self)), "{}\nname={!r}".format(self.__class__.__name__, self.name))
 
+class FunctionCallExpression(ExpressionBase):
+	__slots__ = ('function', 'arguments',)
+	def __init__(self, context, function, arguments):
+		self.context = context
+		self.function = function
+		if self.function.result_type != DataType.UNDEFINED:
+			function_type = self.function.result_type
+			self._validate_function(function_type, arguments)
+			self.result_type = function_type.return_type
+		self.arguments = arguments
+
+	@classmethod
+	def build(cls, context, function, arguments):
+		return cls(context, function.build(), tuple(argument.build() for argument in arguments)).reduce()
+
+	# because there is no syntax for defining a function, they can't be reduced until symbols can be reduced
+
+	def evaluate(self, thing):
+		function = self.function.evaluate(thing)
+		if not callable(function):
+			raise errors.EvaluationError('data type mismatch (not a callable value)')
+		arguments = tuple(argument.evaluate(thing) for argument in self.arguments)
+		function_name = '<unknown>'
+		if self.function.result_type != DataType.UNDEFINED:
+			function_type = self.function.result_type
+			function_name = function_type.value_name
+			self._validate_function(function_type, arguments)
+		elif hasattr(function, '__name__'):
+			function_name = function.__name__ + '?'
+		try:
+			result = function(*arguments)
+		except errors.FunctionCallError as error:
+			error.function_name = function_name
+			raise error
+		except Exception as error:
+			raise errors.FunctionCallError('function call failed', error=error, function_name=function_name) from None
+		return self._new_value(result)
+
+	def _validate_function(self, function_type, arguments):
+		if not isinstance(function_type, DataType.FUNCTION.__class__):
+			raise errors.EvaluationError('data type mismatch (not a callable value)')
+		if function_type.minimum_arguments is not DataType.UNDEFINED:
+			if len(arguments) < function_type.minimum_arguments:
+				raise errors.FunctionCallError(
+					"expected at least {} positional arguments".format(function_type.minimum_arguments),
+					function_name=function_type.value_name
+				)
+		if function_type.argument_types is not DataType.UNDEFINED:
+			if len(arguments) > len(function_type.argument_types):
+				raise errors.FunctionCallError(
+					"expected at most {} positional arguments".format(len(function_type.argument_types)),
+					function_name=function_type.value_name
+				)
+			for pos, (arg1, arg2_type) in enumerate(zip(arguments, function_type.argument_types), 1):
+				if isinstance(arg1, ExpressionBase):
+					arg1_type = arg1.result_type
+				else:
+					arg1_type = DataType.from_value(arg1)
+				if not DataType.is_compatible(arg1_type, arg2_type):
+					raise errors.FunctionCallError(
+						"data type mismatch (argument #{})".format(pos),
+						function_name=function_type.value_name
+					)
+
+	def to_graphviz(self, digraph, *args, **kwargs):
+		super(FunctionCallExpression, self).to_graphviz(digraph, *args, **kwargs)
+		self.function.to_graphviz(digraph, *args, **kwargs)
+		digraph.edge(str(id(self)), str(id(self.function)), label='function')
+		for idx, argument in enumerate(self.arguments, 1):
+			argument.to_graphviz(digraph, *args, **kwargs)
+			digraph.edge(str(id(self)), str(id(argument)), label="argument #{}".format(idx))
+
 class Statement(ASTNodeBase):
 	"""A class representing the top level statement of the grammar text."""
-	__slots__ = ('context', 'expression')
-	def __init__(self, context, expression):
+	__slots__ = ('context', 'expression', 'comment')
+	def __init__(self, context, expression, comment=None):
 		"""
 		:param context: The context to use for evaluating the statement.
 		:type context: :py:class:`~rule_engine.engine.Context`
@@ -1035,10 +1111,11 @@ class Statement(ASTNodeBase):
 		"""
 		self.context = context
 		self.expression = expression
+		self.comment = comment
 
 	@classmethod
-	def build(cls, context, expression):
-		return cls(context, expression.build()).reduce()
+	def build(cls, context, expression, **kwargs):
+		return cls(context, expression.build(), **kwargs).reduce()
 
 	def evaluate(self, thing):
 		return self.expression.evaluate(thing)
@@ -1047,6 +1124,8 @@ class Statement(ASTNodeBase):
 		super(Statement, self).to_graphviz(digraph, *args, **kwargs)
 		self.expression.to_graphviz(digraph, *args, **kwargs)
 		digraph.edge(str(id(self)), str(id(self.expression)))
+		if self.comment:
+			self.comment.to_graphviz(digraph, *args, **kwargs)
 
 class TernaryExpression(ExpressionBase):
 	"""
@@ -1070,6 +1149,7 @@ class TernaryExpression(ExpressionBase):
 			self.result_type = self.case_true.result_type
 		elif isinstance(self.case_true.result_type, DataType.ARRAY.__class__) and isinstance(self.case_false.result_type, DataType.ARRAY.__class__):
 			self.result_type = DataType.ARRAY
+		# todo: the other compound types should be checked here as well.
 
 	@classmethod
 	def build(cls, context, condition, case_true, case_false):
@@ -1080,12 +1160,9 @@ class TernaryExpression(ExpressionBase):
 		return case.evaluate(thing)
 
 	def reduce(self):
-		if _is_reduced(self.condition):
-			reduced_condition = bool(self.condition.value)
-		else:
-			reduced_condition = self.condition.reduce()
-			if reduced_condition is self.condition:
-				return self
+		if not _is_reduced(self.condition):
+			return self
+		reduced_condition = bool(self.condition.value)
 		return self.case_true.reduce() if reduced_condition else self.case_false.reduce()
 
 	def to_graphviz(self, digraph, *args, **kwargs):
@@ -1112,14 +1189,13 @@ class UnaryExpression(ExpressionBase):
 		self.context = context
 		type_ = type_.lower()
 		self.type = type_
-		self._evaluator = getattr(self, '_op_' + type_)
 		if type_ == 'not':
 			self.result_type = DataType.BOOLEAN
 		elif type_ == 'uminus':
 			self.result_type = right.result_type
 		else:
-			raise errors.EvaluationError('unknown unary expression type')
-
+			raise ValueError('unknown unary expression type')
+		self._evaluator = getattr(self, '_op_' + type_)
 		self.right = right
 
 	@classmethod
