@@ -35,7 +35,9 @@ import code
 import io
 import os
 import pprint
+import re
 import sys
+import textwrap
 import traceback
 
 from . import __version__
@@ -97,6 +99,7 @@ def main():
 			)
 		context = console.locals['context']
 		thing = console.locals['thing']
+	debugging = arguments.debug
 
 	while True:
 		try:
@@ -104,6 +107,12 @@ def main():
 		except (EOFError, KeyboardInterrupt):
 			break
 
+		match = re.match(r'\s*#!\s*debug\s*=\s*(\w+)', rule_text)
+		if match:
+			debugging = match.group(1).lower() != 'false'
+			print('# debugging = ' + str(debugging).lower())
+			continue
+
 		try:
 			rule = engine.Rule(rule_text, context=context)
 			result = rule.evaluate(thing)
@@ -111,7 +120,18 @@ def main():
 			print("{}: {}".format(error.__class__.__name__, error.message))
 			if isinstance(error, (errors.AttributeResolutionError, errors.SymbolResolutionError)) and error.suggestion:
 				print("Did you mean '{}'?".format(error.suggestion))
-			if arguments.debug:
+			elif isinstance(error, errors.RegexSyntaxError):
+				print("  Regex:   {!r}".format(error.error.pattern))
+				print("  Details: {} at position {}".format(error.error.msg, error.error.pos))
+			elif isinstance(error, errors.FunctionCallError):
+				if debugging:
+					inner_exception = ''.join(traceback.format_exception(
+						error.error,
+						error.error,
+						error.error.__traceback__
+					))
+					print(textwrap.indent(inner_exception, ' ' * 2))
+			if debugging:
 				traceback.print_exc()
 		except Exception as error:
 			traceback.print_exc()
