@@ -41,18 +41,13 @@ import re
 import threading
 
 from . import ast
+from . import builtins
 from . import errors
 from . import parser
 from .suggestions import suggest_symbol
 
 import dateutil.tz
 
-def _now(builtins):
-	return datetime.datetime.now(tz=builtins.timezone)
-
-def _today(builtins):
-	return _now(builtins).replace(hour=0, minute=0, second=0, microsecond=0)
-
 def _tls_getter(thread_local, key, _builtins):
 	# a function stub to be used with functools.partial for retrieving thread-local values
 	return getattr(thread_local.storage, key)
@@ -275,7 +270,7 @@ class _AttributeResolver(object):
 
 	@attribute('to_ary', ast.DataType.STRING, result_type=ast.DataType.ARRAY(ast.DataType.STRING))
 	def string_to_ary(self, value):
-		return value.split()
+		return tuple(value)
 
 	@attribute('to_flt', ast.DataType.STRING, result_type=ast.DataType.FLOAT)
 	def string_to_flt(self, value):
@@ -306,81 +301,6 @@ class _AttributeResolver(object):
 	def value_to_set(self, value):
 		return set(value)
 
-class Builtins(collections.abc.Mapping):
-	"""
-	A class to define and provide variables to within the builtin context of rules. These can be accessed by specifying
-	a symbol name with the ``$`` prefix.
-	"""
-	scope_name = 'built-in'
-	"""The identity name of the scope for builtin symbols."""
-	def __init__(self, values, namespace=None, timezone=None, value_types=None):
-		"""
-		:param dict values: A mapping of string keys to be used as symbol names with values of either Python literals or
-			a function which will be called when the symbol is accessed. When using a function, it will be passed a
-			single argument, which is the instance of :py:class:`Builtins`.
-		:param str namespace: The namespace of the variables to resolve.
-		:param timezone: A timezone to use when resolving timestamps.
-		:type timezone: :py:class:`~datetime.tzinfo`
-		:param dict value_types: A mapping of the values to their datatypes.
-
-		.. versionchanged:: 2.3.0
-			Added the *value_types* parameter.
-		"""
-		self.__values = values
-		self.__value_types = value_types or {}
-		self.namespace = namespace
-		self.timezone = timezone or dateutil.tz.tzlocal()
-
-	def resolve_type(self, name):
-		"""
-		The method to use for resolving the data type of a builtin symbol.
-
-		:param str name: The name of the symbol to retrieve the data type of.
-		:return: The data type of the symbol or :py:attr:`~rule_engine.ast.DataType.UNDEFINED`.
-		"""
-		return self.__value_types.get(name, ast.DataType.UNDEFINED)
-
-	def __repr__(self):
-		return "<{} namespace={!r} keys={!r} timezone={!r} >".format(self.__class__.__name__, self.namespace, tuple(self.keys()), self.timezone)
-
-	def __getitem__(self, name):
-		value = self.__values[name]
-		if isinstance(value, collections.abc.Mapping):
-			if self.namespace is None:
-				namespace = name
-			else:
-				namespace = self.namespace + '.' + name
-			return self.__class__(value, namespace=namespace, timezone=self.timezone)
-		elif callable(value):
-			value = value(self)
-		return value
-
-	def __iter__(self):
-		return iter(self.__values)
-
-	def __len__(self):
-		return len(self.__values)
-
-	@classmethod
-	def from_defaults(cls, values=None, **kwargs):
-		"""Initialize a :py:class:`Builtins` instance with a set of default values."""
-		# there may be errors here if the decimal.Context precision exceeds what is provided by the math constants
-		default_values = {
-			'e': decimal.Decimal(repr(math.e)),
-			'pi': decimal.Decimal(repr(math.pi)),
-			'now': _now,
-			'today': _today
-		}
-		default_values.update(values or {})
-		default_value_types = {
-			'e': ast.DataType.FLOAT,
-			'pi': ast.DataType.FLOAT,
-			'now': ast.DataType.DATETIME,
-			'today': ast.DataType.DATETIME
-		}
-		default_value_types.update(kwargs.pop('value_types', {}))
-		return cls(default_values, value_types=default_value_types, **kwargs)
-
 class _ThreadLocalStorage(object):
 	"""
 	An object whose attributes are required to be tracked separately among multiple threads. This is to guarantee that
@@ -403,6 +323,7 @@ class Context(object):
 	"""
 	def __init__(
 			self,
+			*,
 			regex_flags=0,
 			resolver=None,
 			type_resolver=None,
@@ -460,12 +381,12 @@ class Context(object):
 		"""The *default_timezone* parameter from :py:meth:`~__init__`"""
 		self.default_value = default_value
 		"""The *default_value* parameter from :py:meth:`~__init__`"""
-		self.builtins = Builtins.from_defaults(
-			values={'re_groups': functools.partial(_tls_getter, self._thread_local, 'regex_groups')},
+		self.builtins = builtins.Builtins.from_defaults(
+			values={'re_groups': builtins.BuiltinValueGenerator(functools.partial(_tls_getter, self._thread_local, 'regex_groups'))},
 			value_types={'re_groups': ast.DataType.ARRAY(ast.DataType.STRING)},
 			timezone=default_timezone
 		)
-		"""An instance of :py:class:`Builtins` to provided a default set of builtin symbol values."""
+		"""An instance of :py:class:`~rule_engine.builtins.Builtins` to provided a default set of builtin symbol values."""
 		self.decimal_context = decimal_context or decimal.getcontext()
 		"""The *decimal_context* parameter from :py:meth:`~__init__`"""
 		if isinstance(type_resolver, collections.abc.Mapping):
@@ -479,8 +400,8 @@ class Context(object):
 		Add the specified assignments to a thread-specific scope. This is used when an assignment originates from an
 		expression.
 
-		:param assignment: The one or more assignments to define.
-		:type assignment: :py:class:`~rule_engine.ast.Assignment`
+		:param assignments: The one or more assignments to define.
+		:type assignments: :py:class:`~rule_engine.ast.Assignment`
 		"""
 		self._tls.assignment_scopes.append({assign.name: assign for assign in assignments})
 		try:
@@ -508,9 +429,9 @@ class Context(object):
 		:param str name: The symbol name that is being resolved.
 		:return: The value for the corresponding symbol *name*.
 		"""
-		if scope == Builtins.scope_name:
+		if scope == builtins.Builtins.scope_name:
 			thing = self.builtins
-		if isinstance(thing, Builtins):
+		if isinstance(thing, builtins.Builtins):
 			return resolve_item(thing, name)
 		if scope is None:
 			for assignments in self._tls.assignment_scopes:
@@ -547,7 +468,7 @@ class Context(object):
 		:param str scope: An optional scope name that identifies from where to resolve the name.
 		:return: The type of the specified symbol.
 		"""
-		if scope == Builtins.scope_name:
+		if scope == builtins.Builtins.scope_name:
 			return self.builtins.resolve_type(name)
 		for assignments in self._tls.assignment_scopes:
 			if name in assignments:
