@@ -77,8 +77,27 @@ class DatetimeSyntaxError(SyntaxError):
 		self.value = value
 		"""The datetime value which contains the syntax error which caused this exception to be raised."""
 
+class FloatSyntaxError(SyntaxError):
+	"""
+	An error raised for issues regarding the use of improperly formatted float expressions.
+
+	.. versionadded:: 4.0.0
+	"""
+	def __init__(self, message, value):
+		"""
+		:param str message: A text description of what error occurred.
+		:param str value: The float value which contains the syntax error which caused this exception to be raised.
+		"""
+		super(FloatSyntaxError, self).__init__(message)
+		self.value = value
+		"""The float value which contains the syntax error which caused this exception to be raised."""
+
 class TimedeltaSyntaxError(SyntaxError):
-	"""An error raised for issues regarding the use of improperly formatted timedelta expressions."""
+	"""
+	An error raised for issues regarding the use of improperly formatted timedelta expressions.
+
+	.. versionadded:: 3.5.0
+	"""
 	def __init__(self, message, value):
 		"""
 		:param str message: A text description of what error occurred.
@@ -105,7 +124,7 @@ class RegexSyntaxError(SyntaxError):
 		"""The regular expression value which contains the syntax error which caused this exception to be raised."""
 
 class RuleSyntaxError(SyntaxError):
-	"""An error raised for issues identified in while parsing the grammar of the rule text."""
+	"""An error raised for issues identified while parsing the grammar of the rule text."""
 	def __init__(self, message, token=None):
 		"""
 		:param str message: A text description of what error occurred.
@@ -248,3 +267,15 @@ class SymbolTypeError(EvaluationError):
 			expected_type.name
 		)
 		super(SymbolTypeError, self).__init__(message)
+
+class FunctionCallError(EvaluationError):
+	"""
+	An error raised when there is an issue calling a function.
+
+	.. versionadded:: 4.0.0
+	"""
+	def __init__(self, message, error=None, function_name=None):
+		super(FunctionCallError, self).__init__(message)
+		self.error = error
+		"""The exception from which this error was triggered."""
+		self.function_name = function_name
