@@ -156,7 +156,7 @@ class Parser(ParserBase):
 	tokens = (
 		'DATETIME', 'TIMEDELTA', 'FLOAT', 'STRING', 'SYMBOL',
 		'LPAREN', 'RPAREN', 'QMARK', 'COLON', 'COMMA',
-		'LBRACKET', 'RBRACKET', 'LBRACE', 'RBRACE'
+		'LBRACKET', 'RBRACKET', 'LBRACE', 'RBRACE', 'COMMENT'
 	) + tuple(set(list(reserved_words.values()) + list(op_names.values())))
 
 	t_ignore = ' \t'
@@ -183,25 +183,31 @@ class Parser(ParserBase):
 	t_ATTR             = r'(?<=\S)\.(?=[a-zA-Z_][a-zA-Z0-9_]*)'
 	t_ATTR_SAFE        = r'(?<=\S)&\.(?=[a-zA-Z_][a-zA-Z0-9_]*)'
 
-	# tokens are listed from lowest to highest precedence, ones that appear
-	# later are effectively evaluated first
-	# see: https://en.wikipedia.org/wiki/Order_of_operations#Programming_languages
-	precedence = (
-		('left',     'OR'),
-		('left',     'AND'),
-		('right',    'NOT'),
-		('left',     'BWOR'),
-		('left',     'BWXOR'),
-		('left',     'BWAND'),
-		('right',    'QMARK', 'COLON'),
-		('nonassoc', 'EQ', 'NE', 'EQ_FZM', 'EQ_FZS', 'NE_FZM', 'NE_FZS', 'GE', 'GT', 'LE', 'LT', 'IN'),  # Nonassociative operators
-		('left',     'ADD', 'SUB'),
-		('left',     'BWLSH', 'BWRSH'),
-		('left',     'MUL', 'TDIV', 'FDIV', 'MOD'),
-		('left',     'POW'),
-		('right',    'UMINUS'),
-		('left',     'ATTR', 'ATTR_SAFE'),
-	)
+	# Tokens are listed from highest to lowest precedence, ones that appear
+	# later are effectively evaluated last
+	# see:
+	#   * https://en.wikipedia.org/wiki/Order_of_operations#Programming_languages
+	#   * https://docs.python.org/3/reference/expressions.html
+	precedence = tuple(
+		# reverse the order to lowest to highest for ply
+		reversed((
+			('nonassoc', 'LPAREN', 'RPAREN'),
+			('left',     'ATTR', 'ATTR_SAFE'),
+			('right',    'UMINUS'),
+			('left',     'POW'),
+			('left',     'MUL', 'TDIV', 'FDIV', 'MOD'),
+			('left',     'BWLSH', 'BWRSH'),
+			('left',     'ADD', 'SUB'),
+			('nonassoc', 'EQ', 'NE', 'EQ_FZM', 'EQ_FZS', 'NE_FZM', 'NE_FZS', 'GE', 'GT', 'LE', 'LT', 'IN'),  # Nonassociative operators
+			('right',    'QMARK', 'COLON'),
+			('left',     'BWAND'),
+			('left',     'BWXOR'),
+			('left',     'BWOR'),
+			('right',    'NOT'),
+			('left',     'AND'),
+			('left',     'OR'),
+		)
+	))
 
 	@classmethod
 	def get_token_regex(cls, token_name):
@@ -275,6 +281,10 @@ class Parser(ParserBase):
 		t.type = self.reserved_words.get(t.value, 'SYMBOL')
 		return t
 
+	def t_COMMENT(self, t):
+		r'\#.*$'
+		return t
+
 	def t_newline(self, t):
 		r'\n+'
 		t.lexer.lineno += t.value.count("\n")
@@ -287,8 +297,14 @@ class Parser(ParserBase):
 		raise errors.RuleSyntaxError('syntax error', token)
 
 	def p_statement_expr(self, p):
-		'statement : expression'
-		p[0] = _DeferredAstNode(ast.Statement, args=(self.context, p[1]))
+		"""
+		statement : expression
+		          | expression COMMENT
+		"""
+		kwargs = {}
+		if len(p) == 3:
+			kwargs['comment'] = ast.Comment(p[2][1:].strip())
+		p[0] = _DeferredAstNode(ast.Statement, args=(self.context, p[1]), kwargs=kwargs)
 
 	def p_expression_getattr(self, p):
 		"""
@@ -450,16 +466,13 @@ class Parser(ParserBase):
 		p[0] = _DeferredAstNode(ast.TimedeltaExpression, args=(self.context, p[1]), method='from_string')
 
 	def p_expression_float(self, p):
-		'expression : FLOAT'
-		p[0] = _DeferredAstNode(ast.FloatExpression, args=(self.context, float(literal_eval(p[1]))))
-
-	def p_expression_float_nan(self, p):
-		'expression : FLOAT_NAN'
-		p[0] = _DeferredAstNode(ast.FloatExpression, args=(self.context, float('nan')))
-
-	def p_expression_float_inf(self, p):
-		'expression : FLOAT_INF'
-		p[0] = _DeferredAstNode(ast.FloatExpression, args=(self.context, float('inf')))
+		"""
+		expression : FLOAT
+				   | FLOAT_NAN
+				   | FLOAT_INF
+		"""
+		str_val = p[1]
+		p[0] = _DeferredAstNode(ast.FloatExpression, args=(self.context, str_val), method='from_string')
 
 	def p_expression_null(self, p):
 		'object : NULL'
@@ -563,3 +576,17 @@ class Parser(ParserBase):
 		else:
 			raise errors.RuleSyntaxError('invalid get slice expression')
 		p[0] = _DeferredAstNode(ast.GetSliceExpression, args=(self.context, container, start, stop), kwargs={'safe': safe})
+
+	def p_expression_function_call(self, p):
+		"""
+		object : expression LPAREN RPAREN
+		       | expression LPAREN ary_members RPAREN
+		"""
+		function = p[1]
+		if len(p) == 4:
+			arguments = collections.deque()
+		elif len(p) == 5:
+			arguments = p[3]
+		else:
+			raise errors.RuleSyntaxError('invalid function call expression')
+		p[0] = _DeferredAstNode(ast.FunctionCallExpression, args=(self.context, function, arguments))
