@@ -49,6 +49,7 @@ __all__ = (
 	'iterable_member_value_type'
 )
 
+_PYTHON_FUNCTION_TYPE = type(lambda: None)
 NoneType = type(None)
 
 def _to_decimal(value):
@@ -181,6 +182,9 @@ class _DataTypeDef(object):
 		self.name = name
 		self.python_type = python_type
 		self.is_scalar = True
+		if '__call__' in dir(self) and self.__call__.__doc__:
+			# patch the call docs into the top-level class for Sphinx
+			self.__class__.__doc__ = self.__call__.__doc__
 
 	@property
 	def is_iterable(self):
@@ -201,7 +205,12 @@ class _DataTypeDef(object):
 	def is_compound(self):
 		return not self.is_scalar
 
-_DATA_TYPE_UNDEFINED = _DataTypeDef('UNDEFINED', errors.UNDEFINED)
+class _UndefinedDataTypeDef(_DataTypeDef):
+	def __repr__(self):
+		return 'UNDEFINED'
+
+_DATA_TYPE_UNDEFINED = _UndefinedDataTypeDef('UNDEFINED', errors.UNDEFINED)
+
 class _CollectionDataTypeDef(_DataTypeDef):
 	__slots__ = ('value_type', 'value_type_nullable')
 	def __init__(self, name, python_type, value_type=_DATA_TYPE_UNDEFINED, value_type_nullable=True):
@@ -222,6 +231,10 @@ class _CollectionDataTypeDef(_DataTypeDef):
 		return self.value_type
 
 	def __call__(self, value_type, value_type_nullable=True):
+		"""
+		:param value_type: The type of the members.
+		:param bool value_type_nullable: Whether or not members are allowed to be :py:attr:`.NULL`.
+		"""
 		return self.__class__(
 			self.name,
 			self.python_type,
@@ -274,6 +287,11 @@ class _MappingDataTypeDef(_DataTypeDef):
 		return self.key_type
 
 	def __call__(self, key_type, value_type=_DATA_TYPE_UNDEFINED, value_type_nullable=True):
+		"""
+		:param key_type: The type of the mapping keys.
+		:param value_type: The type of the mapping values.
+		:param bool value_type_nullable: Whether or not mapping values are allowed to be :py:attr:`.NULL`.
+		"""
 		return self.__class__(
 			self.name,
 			self.python_type,
@@ -303,6 +321,66 @@ class _MappingDataTypeDef(_DataTypeDef):
 	def __hash__(self):
 		return hash((self.python_type, self.is_scalar, hash((self.key_type, self.value_type, self.value_type_nullable))))
 
+class _FunctionDataTypeDef(_DataTypeDef):
+	__slots__ = ('value_name', 'return_type', 'argument_types', 'minimum_arguments')
+	def __init__(self, name, python_type, value_name=None, return_type=_DATA_TYPE_UNDEFINED, argument_types=_DATA_TYPE_UNDEFINED, minimum_arguments=None):
+		super(_FunctionDataTypeDef, self).__init__(name, python_type)
+		self.value_name = value_name
+		self.return_type = return_type
+		if argument_types is _DATA_TYPE_UNDEFINED:
+			if minimum_arguments is None:
+				minimum_arguments = _DATA_TYPE_UNDEFINED
+		else:
+			if not isinstance(argument_types, collections.abc.Sequence):
+				raise TypeError('argument_types must be a sequence (list or tuple)')
+			if minimum_arguments is None:
+				# if arguments are specified, assume that they're all required by default
+				minimum_arguments = len(argument_types)
+			if len(argument_types) < minimum_arguments:
+				raise ValueError('minimum_arguments can not be greater than the length of argument_types')
+		self.argument_types = argument_types
+		self.minimum_arguments = minimum_arguments
+
+	def __call__(self, name, return_type=_DATA_TYPE_UNDEFINED, argument_types=_DATA_TYPE_UNDEFINED, minimum_arguments=None):
+		"""
+		.. versionadded:: 4.0.0
+
+		:param str name: The name of the function, e.g. "split".
+		:param return_type: The data type of the functions return value.
+		:param tuple argument_types: The data types of the functions arguments.
+		:param int minimum_arguments: The minimum number of arguments the function requires.
+
+		If *argument_types* is specified and *minimum_arguments* is not, then *minimum_arguments* will default to the length
+		of *argument_types* effectively meaning that every defined argument is required. If
+		"""
+		return self.__class__(
+			self.name,
+			self.python_type,
+			value_name=name,
+			return_type=return_type,
+			argument_types=argument_types,
+			minimum_arguments=minimum_arguments
+		)
+	def __repr__(self):
+		return "<{} name={} python_type={} return_type={} >".format(
+			self.__class__.__name__,
+			self.name,
+			self.python_type.__name__,
+			self.return_type.name
+		)
+
+	def __eq__(self, other):
+		if not super().__eq__(other):
+			return False
+		return all((
+			self.return_type == other.return_type,
+			self.argument_types == other.argument_types,
+			self.minimum_arguments == other.minimum_arguments
+		))
+
+	def __hash__(self):
+		return hash((self.python_type, self.is_scalar, hash((self.return_type, self.argument_types, self.minimum_arguments))))
+
 class DataTypeMeta(type):
 	def __new__(metacls, cls, bases, classdict):
 		data_type = super().__new__(metacls, cls, bases, classdict)
@@ -354,34 +432,16 @@ class DataType(metaclass=DataTypeMeta):
 	  This checks that the types are compatible without any kind of conversion. When dealing with compound data types,
 	  this ensures that the member types are either the same or :py:attr:`~.UNDEFINED`.
 	"""
-	ARRAY = _ArrayDataTypeDef('ARRAY', tuple)
-	"""
-	.. py:function:: __call__(value_type, value_type_nullable=True)
-	
-	:param value_type: The type of the array members.
-	:param bool value_type_nullable: Whether or not array members are allowed to be :py:attr:`.NULL`.
-	"""
+	ARRAY = staticmethod(_ArrayDataTypeDef('ARRAY', tuple))
 	BOOLEAN = _DataTypeDef('BOOLEAN', bool)
 	DATETIME = _DataTypeDef('DATETIME', datetime.datetime)
-	TIMEDELTA = _DataTypeDef('TIMEDELTA', datetime.timedelta)
 	FLOAT = _DataTypeDef('FLOAT', decimal.Decimal)
-	MAPPING = _MappingDataTypeDef('MAPPING', dict)
-	"""
-	.. py:function:: __call__(key_type, value_type, value_type_nullable=True)
-	
-	:param key_type: The type of the mapping keys.
-	:param value_type: The type of the mapping values.
-	:param bool value_type_nullable: Whether or not mapping values are allowed to be :py:attr:`.NULL`.
-	"""
+	FUNCTION = staticmethod(_FunctionDataTypeDef('FUNCTION', _PYTHON_FUNCTION_TYPE))
+	MAPPING = staticmethod(_MappingDataTypeDef('MAPPING', dict))
 	NULL = _DataTypeDef('NULL', NoneType)
-	SET = _SetDataTypeDef('SET', set)
-	"""
-	.. py:function:: __call__(value_type, value_type_nullable=True)
-
-	:param value_type: The type of the set members.
-	:param bool value_type_nullable: Whether or not set members are allowed to be :py:attr:`.NULL`.
-	"""
+	SET = staticmethod(_SetDataTypeDef('SET', set))
 	STRING = _DataTypeDef('STRING', str)
+	TIMEDELTA = _DataTypeDef('TIMEDELTA', datetime.timedelta)
 	UNDEFINED = _DATA_TYPE_UNDEFINED
 	"""
 	Undefined values. This constant can be used to indicate that a particular symbol is valid, but it's data type is
@@ -407,15 +467,18 @@ class DataType(metaclass=DataTypeMeta):
 	@classmethod
 	def from_type(cls, python_type):
 		"""
-		Get the supported data type constant for the specified Python type. If the type can not be mapped to a supported
-		data type, then a :py:exc:`ValueError` exception will be raised. This function will not return
-		:py:attr:`.UNDEFINED`.
+		Get the supported data type constant for the specified Python type/type hint. If the type or typehint can not be
+		mapped to a supported data type, then a :py:exc:`ValueError` exception will be raised. This function will not
+		return :py:attr:`.UNDEFINED`.
 
-		:param type python_type: The native Python type to retrieve the corresponding type constant for.
+		:param type python_type: The native Python type or type hint to retrieve the corresponding type constant for.
 		:return: One of the constants.
+
+		.. versionchanged:: 4.1.0
+			Added support for typehints.
 		"""
-		if not isinstance(python_type, type):
-			raise TypeError('from_type argument 1 must be type, not ' + type(python_type).__name__)
+		if not (isinstance(python_type, type) or hasattr(python_type, '__origin__')):
+			raise TypeError('from_type argument 1 must be a type or a type hint, not ' + type(python_type).__name__)
 		if python_type in (list, range, tuple):
 			return cls.ARRAY
 		elif python_type is bool:
@@ -434,6 +497,21 @@ class DataType(metaclass=DataTypeMeta):
 			return cls.SET
 		elif python_type is str:
 			return cls.STRING
+		elif python_type is _PYTHON_FUNCTION_TYPE:
+			return cls.FUNCTION
+		elif hasattr(python_type, "__origin__"):
+			origin_python_type = python_type.__origin__
+			maintype = cls.from_type(origin_python_type)
+			if origin_python_type in (list, tuple, set):
+				if hasattr(python_type, "__args__") and origin_python_type is not tuple:
+					valuetype = cls.from_type(python_type.__args__[0])
+					return maintype(valuetype)
+			if origin_python_type is dict:
+				if hasattr(python_type, "__args__"):
+					key_type = cls.from_type(python_type.__args__[0])
+					value_type = cls.from_type(python_type.__args__[1])
+					return maintype(key_type, value_type)
+			return maintype
 		raise ValueError("can not map python type {0!r} to a compatible data type".format(python_type.__name__))
 
 	@classmethod
@@ -467,6 +545,8 @@ class DataType(metaclass=DataTypeMeta):
 			)
 		elif isinstance(python_value, collections.abc.Sequence):
 			return cls.ARRAY(value_type=iterable_member_value_type(python_value))
+		elif callable(python_value):
+			return cls.FUNCTION
 		raise TypeError("can not map python type {0!r} to a compatible data type".format(type(python_value).__name__))
 
 	@classmethod
@@ -488,6 +568,18 @@ class DataType(metaclass=DataTypeMeta):
 		if dt1 is _DATA_TYPE_UNDEFINED or dt2 is _DATA_TYPE_UNDEFINED:
 			return True
 		if dt1.is_scalar and dt2.is_scalar:
+			if isinstance(dt1, DataType.FUNCTION.__class__) and isinstance(dt2, DataType.FUNCTION.__class__):
+				if not cls.is_compatible(dt1.return_type, dt2.return_type):
+					return False
+				if dt1.argument_types != _DATA_TYPE_UNDEFINED and dt2.argument_types != _DATA_TYPE_UNDEFINED:
+					if len(dt1.argument_types) != len(dt2.argument_types):
+						return False
+					if not all(cls.is_compatible(arg1_dt, arg2_dt) for (arg1_dt, arg2_dt) in zip(dt1.argument_types, dt2.argument_types)):
+						return False
+				if dt1.minimum_arguments != _DATA_TYPE_UNDEFINED and dt2.minimum_arguments != _DATA_TYPE_UNDEFINED:
+					if dt1.minimum_arguments != dt2.minimum_arguments:
+						return False
+				return True
 			return dt1 == dt2
 		elif dt1.is_compound and dt2.is_compound:
 			if isinstance(dt1, DataType.ARRAY.__class__) and isinstance(dt2, DataType.ARRAY.__class__):
