@@ -34,6 +34,8 @@ import datetime
 import decimal
 import itertools
 import math
+import random
+import string
 import unittest
 
 import rule_engine.ast as ast
@@ -58,6 +60,15 @@ class ParserTestsBase(unittest.TestCase):
 		return statement
 
 class ParserTests(ParserTestsBase):
+	def test_parser_comment_expressions(self):
+		expression = self.assertStatementType('null', ast.NullExpression)
+		self.assertIsNone(expression.comment)
+
+		comment = ''.join(random.choice(string.ascii_letters) for _ in range(10))
+		expression = self.assertStatementType('null # ' + comment, ast.NullExpression)
+		self.assertIsInstance(expression.comment, ast.Comment)
+		self.assertEqual(expression.comment.value, comment)
+
 	def test_parser_comprehension_expressions(self):
 		expressions = []
 		# conditional
@@ -117,6 +128,23 @@ class ParserTests(ParserTestsBase):
 			self.assertIsInstance(expression.container, ast.SymbolExpression)
 			self.assertEqual(expression.container.name, 'container')
 
+	def test_parser_function_call_expressions(self):
+		expression = self.assertStatementType(
+			'function()',
+			ast.FunctionCallExpression
+		).expression
+		self.assertIsInstance(expression.function, ast.SymbolExpression)
+		self.assertEqual(expression.function.name, 'function')
+		self.assertEqual(len(expression.arguments), 0)
+
+		expression = self.assertStatementType(
+			'function(arg1)',
+			ast.FunctionCallExpression
+		).expression
+		self.assertEqual(len(expression.arguments), 1)
+		self.assertIsInstance(expression.arguments[0], ast.SymbolExpression)
+		self.assertEqual(expression.arguments[0].name, 'arg1')
+
 	def test_parser_order_of_operations(self):
 		cases = (
 			'100 * ( 2 + 12 ) / 14',
@@ -343,8 +371,10 @@ class ParserLiteralTests(ParserTestsBase):
 		self.assertLiteralStatementEqual('0o77', ast.FloatExpression, decimal.Decimal('63'))
 
 	def test_parse_float_base_10(self):
-		self.assertLiteralStatementEqual('00', ast.FloatExpression, decimal.Decimal('0'))
+		self.assertLiteralStatementEqual('0', ast.FloatExpression, decimal.Decimal('0'))
 		self.assertLiteralStatementEqual('99', ast.FloatExpression, decimal.Decimal('99'))
+		with self.assertRaises(errors.FloatSyntaxError):
+			self._parse('00', self.context)
 
 	def test_parse_float_base_16(self):
 		self.assertLiteralStatementEqual('0x00', ast.FloatExpression, decimal.Decimal('0'))
